from django import forms


class AppForm(forms.Form):
    name = forms.CharField(max_length=60)
    platform = forms.ChoiceField()
    plan = forms.ChoiceField(required=False)
    teamOwner = forms.ChoiceField(required=False)
    pool = forms.ChoiceField(required=False)



from django.test import TestCase
from django.core.urlresolvers import reverse


class TokeRequestSuccessViewTest(TestCase):
    def test_get(self):
        response = self.client.get(reverse('token-request-success'))
        self.assertEqual(200, response.status_code)
        self.assertTemplateUsed('auth/token_request_success.html')



from mock import patch, Mock

from django.test import TestCase
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse

from tsuru_dashboard import settings
from tsuru_dashboard.services.views import Bind


class BindViewTest(TestCase):
    @patch("requests.put")
    @patch("requests.get")
    def test_view(self, get, put):
        get.return_value = Mock(status_code=200)
        app = "app"
        request = RequestFactory().post("/", {"app": app})
        request.session = {"tsuru_token": "admin"}
        instance = "service"
        service = "service"

        response = Bind.as_view()(request, service=service, instance=instance)

        self.assertEqual(302, response.status_code)
        url = reverse('service-detail', args=[service, instance])
        self.assertEqual(url, response.items()[1][1])
        url = '{}/services/{}/instances/{}/{}'.format(settings.TSURU_HOST, service, instance, app)
        put.assert_called_with(url, headers={'authorization': 'admin'})



from django.test import TestCase
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse

from tsuru_dashboard import settings
from tsuru_dashboard.teams.views import AddUser

from mock import patch, Mock


class AddUserViewTest(TestCase):
    def setUp(self):
        data = {"user": "username"}
        self.request = RequestFactory().post("/", data)
        self.request.session = {"tsuru_token": "admin"}

    @patch("django.contrib.messages.error")
    @patch("requests.put")
    @patch("requests.get")
    def test_view(self, get, put, error):
        get.return_value = Mock(status_code=200)
        team_name = "avengers"
        response = AddUser.as_view()(self.request, team=team_name)
        self.assertEqual(302, response.status_code)
        url = reverse("team-info", args=[team_name])
        self.assertEqual(url, response.items()[1][1])
        url = "{0}/teams/{1}/{2}".format(settings.TSURU_HOST, team_name,
                                         "username")
        headers = {"authorization": "admin"}
        put.assert_called_with(url, headers=headers)



from mock import patch, Mock

from django.test import TestCase
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse

from tsuru_dashboard import settings
from tsuru_dashboard.auth.views import LoginRequiredView
from tsuru_dashboard.teams.views import Add
from tsuru_dashboard.teams.forms import TeamForm


class TeamViewTest(TestCase):
    @patch("requests.get")
    def setUp(self, get):
        get.return_value = Mock(status_code=200)
        self.factory = RequestFactory()
        self.request = self.factory.get('/')
        self.request.session = {"tsuru_token": "admin"}
        self.response = Add.as_view()(self.request)
        self.request_post = self.factory.post('/team/', {'name': 'test-team'})
        self.request_post.session = {"tsuru_token": "admin"}

    def test_should_require_login_to_create_team(self):
        assert issubclass(Add, LoginRequiredView)

    def test_team_should_render_expected_template(self):
        self.assertEqual('auth/team.html', self.response.template_name)

    def test_context_should_contain_form(self):
        self.assertIn('form', self.response.context_data.keys())

    def test_form_in_context_should_has_a_instance_of_TeamForm(self):
        form = self.response.context_data.get('form')
        self.assertIsInstance(form, TeamForm)

    def test_get_request_team_url_should_not_return_404(self):
        response = self.client.get(reverse('team-add'))
        self.assertNotEqual(404, response.status_code)

    @patch('requests.post')
    @patch('requests.get')
    def test_post_sends_request_to_tsuru(self, get, post):
        get.return_value = Mock(status_code=200)
        self.request_post.session = {'tsuru_token': 'tokentest'}
        Add.as_view()(self.request_post)
        self.assertEqual(1, post.call_count)
        post.assert_called_with(
            '%s/teams' % settings.TSURU_HOST,
            data={"name": "test-team"},
            headers={'authorization':
                     self.request_post.session['tsuru_token']})

    @patch('requests.post')
    @patch('requests.get')
    def test_valid_post_redirect_to_team_list(self, get, post):
        get.return_value = Mock(status_code=200)
        post.return_value = Mock(status_code=200)
        response = Add.as_view()(self.request_post)
        self.assertEqual(302, response.status_code)
        self.assertEqual(reverse('team-list'), response.items()[1][1])

    @patch('requests.post')
    @patch('requests.get')
    def test_post_with_invalid_name_should_return_500(self, get, post):
        get.return_value = Mock(status_code=200)
        post.return_value = Mock(status_code=500, content='Error')
        response = Add.as_view()(self.request_post)
        self.assertEqual('Error', response.context_data.get('errors'))

    @patch("requests.get")
    def test_post_without_name_should_return_form_with_errors(self, get):
        get.return_value = Mock(status_code=200)
        request = self.factory.post('/team/', {'name': ''})
        request.session = {"tsuru_token": "admin"}
        response = Add.as_view()(request)
        form = response.context_data.get('form')
        self.assertIn('name', form.errors)
        self.assertIn(u'This field is required.', form.errors.get('name'))



from django.test import TestCase
from tsuru_dashboard.metrics.backends.elasticsearch import AppFilter, ComponentFilter, NodeFilter


class ElasticSearchFilterTest(TestCase):
    def test_app_filters(self):
        expected_filter = {
            "bool": {
                "must": [
                    {
                        "range": {
                            "@timestamp": {
                                "gte": "now-1h",
                                "lt": "now"
                            }
                        }
                    },
                    {
                        "term": {"app.raw": "app_name"}
                    },
                    {
                        "term": {"process.raw": "process_name"}
                    }
                ]
            }
        }
        filter = AppFilter(app="app_name", process_name="process_name").filter
        self.assertDictEqual(filter, expected_filter)

    def test_component_filters(self):
        expected_filter = {
            "bool": {
                "must": [
                    {
                        "range": {
                            "@timestamp": {
                                "gte": "now-1h",
                                "lt": "now"
                            }
                        }
                    },
                    {
                        "term": {"container.raw": "comp_name"}
                    },
                ]
            }
        }
        filter = ComponentFilter(component="comp_name").filter
        self.assertDictEqual(filter, expected_filter)

    def test_node_filters(self):
        expected_filter = {
            "bool": {
                "must": [
                    {
                        "range": {
                            "@timestamp": {
                                "gte": "now-1h",
                                "lt": "now"
                            }
                        }
                    },
                    {
                        "bool": {
                            "should": [
                                {"terms": {"addr.raw": ["127.0.0.1"]}},
                            ]
                        },
                    },
                ]
            }
        }
        filter = NodeFilter(node="127.0.0.1").filter
        self.assertDictEqual(filter, expected_filter)



from django.test import TestCase
from django.test.client import RequestFactory
from mock import Mock, patch

from tsuru_dashboard import settings
from tsuru_dashboard.auth.forms import SignupForm
from tsuru_dashboard.auth.views import Signup


class SignupViewTest(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.request = self.factory.get('/')
        self.response = Signup().get(self.request)
        self.response_mock = Mock()

    def test_signup_should_show_template(self):
        self.assertEqual('auth/signup.html', self.response.template_name)

    def test_context_should_contain_form(self):
        form = self.response.context_data['signup_form']
        self.assertIsInstance(form, SignupForm)

    def test_should_validate_data_from_post(self):
        data = {'email': '', 'password': '', 'same_password_again': ''}
        request = self.factory.post('/signup', data)
        response = Signup().post(request)
        form = response.context_data['signup_form']
        self.assertIn(u'This field is required.', form.errors['email'])
        self.assertIn(u'This field is required.', form.errors['password'])
        self.assertIn(u'This field is required.',
                      form.errors['same_password_again'])

    @patch('requests.post')
    @patch('requests.get')
    def test_return_msg_on_success(self, get, post):
        get.return_value = Mock(status_code=200)
        data = {'email': 'test@test.com', 'password': 'abc123',
                'same_password_again': 'abc123'}
        request = self.factory.post('/signup', data)
        post.return_value = Mock(status_code=201)
        response = Signup.as_view()(request)
        expected = 'User "{0}" successfully created!'.format(data["email"])
        self.assertEqual(expected, response.context_data["message"])

    @patch('requests.post')
    @patch('requests.get')
    def test_post_sends_to_tsuru_with_args_expected(self, get, post):
        get.return_value = Mock(status_code=200)
        data = {'email': 'test@test.com', 'password': 'abc123',
                'same_password_again': 'abc123'}
        request = self.factory.post('/signup', data)
        Signup().post(request)
        self.assertEqual(1, post.call_count)
        post.assert_called_with(
            '%s/users' % settings.TSURU_HOST,
            data='{"password": "abc123", "email": "test@test.com"}')



from django.http import HttpResponse, HttpResponseBadRequest
from urlparse import urlparse

from tsuru_dashboard import settings
from tsuru_dashboard.auth.views import LoginRequiredView
from .backends.elasticsearch import NodeMetricsBackend, NodesMetricsBackend
from .backends import get_app_backend, get_tsuru_backend
import json
import requests


class Metric(LoginRequiredView):
    def get(self, *args, **kwargs):
        token = self.request.session.get('tsuru_token')
        metric = self.request.GET.get("metric")
        if not metric:
            return HttpResponseBadRequest()

        interval = self.request.GET.get("interval")
        date_range = self.request.GET.get("date_range")
        target = kwargs['target']

        backends = self.get_metrics_backend(metric=metric, target=target, date_range=date_range, token=token)
        if backends is None:
            return HttpResponseBadRequest()

        if not isinstance(backends, list):
            backends = [backends]

        data = {}
        for backend in backends:
            try:
                data = getattr(backend, metric)(interval=interval)
            except:
                continue
            if isinstance(data, dict) and len(data["data"]) > 0:
                data["source"] = backend.url
                break

        return HttpResponse(json.dumps(data))


class AppMetric(Metric):
    def get_metrics_backend(self, metric, target, date_range, token):
        process_name = self.request.GET.get("process_name")
        return get_app_backend(app_name=target, token=token, date_range=date_range, process_name=process_name)


class ComponentMetric(Metric):
    def get_metrics_backend(self, metric, target, date_range, token):
        return get_tsuru_backend(component=target, token=token, date_range=date_range)


class NodeMetric(Metric):
    def get_metrics_backend(self, metric, target, date_range, token):
        return NodeMetricsBackend(addr=target, date_range=date_range)


class PoolMetric(Metric):
    def extract_ip(self, address):
        if not urlparse(address).scheme:
            address = "http://"+address
        return urlparse(address).hostname

    def get_pool_nodes(self, pool_name):
        url = "{}/docker/node".format(settings.TSURU_HOST)
        response = requests.get(url, headers=self.authorization)
        pool_nodes = []
        if response.status_code != 204:
            nodes = response.json().get("nodes", [])

            for node in nodes:
                if node["Metadata"].get("pool", "") == pool_name:
                    addr = self.extract_ip(node["Address"])
                    pool_nodes.append(addr)

        return pool_nodes

    def get_metrics_backend(self, metric, target, date_range, token):
        addrs = self.get_pool_nodes(target)
        return NodesMetricsBackend(addrs=addrs, date_range=date_range)



from django.test import TestCase
from django.test.client import RequestFactory
from tsuru_dashboard import settings
from tsuru_dashboard.metrics import views
from mock import patch
import httpretty
import json


class AppMetricViewTest(TestCase):
    def request(self, string_params=""):
        request = RequestFactory().get("/ble/" + string_params)
        request.session = {"tsuru_token": "token"}
        return request

    @patch("tsuru_dashboard.metrics.backends.get_envs")
    @patch("tsuru_dashboard.metrics.backends.get_app")
    @patch("requests.get")
    @patch("requests.post")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_get_app_metric(self, token_mock, get_mock, post_mock, get_app, get_envs):
        get_app.return_value = {"name": "app_name"}
        get_envs.return_value = {"METRICS_ELASTICSEARCH_HOST": "http://easearch.com"}

        token_mock.return_value = True
        view = views.AppMetric.as_view()

        request = self.request("?metric=cpu_max&date_range=2h/h&interval=30m&process_name=web")
        response = view(request, target="app_name")
        self.assertEqual(response.status_code, 200)


class ComponentMetricViewTest(TestCase):
    def request(self, string_params=""):
        request = RequestFactory().get("/ble/" + string_params)
        request.session = {"tsuru_token": "token"}
        return request

    @patch("requests.post")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_get_component_metric(self, token_mock, post_mock):
        token_mock.return_value = True
        view = views.ComponentMetric.as_view()

        request = self.request("?metric=cpu_max&date_range=2h/h&interval=30m")
        response = view(request, target="comp_name")

        self.assertEqual(response.status_code, 200)


class MetricViewTest(TestCase):
    def request(self, string_params=""):
        request = RequestFactory().get("/ble/" + string_params)
        request.session = {"tsuru_token": "token"}
        return request

    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_get_bad_request(self, token_mock):
        request = RequestFactory().get("")
        request.session = {"tsuru_token": "token"}
        token_mock.return_value = True
        view = views.Metric.as_view()

        response = view(request, name="app_name")

        self.assertEqual(response.status_code, 400)


class PoolMetricViewTest(TestCase):

    def setUp(self):
        self.api_url = "{}/docker/node".format(settings.TSURU_HOST)
        data = {
            "machines": None,
            "nodes": [
                {"Address": "http://128.0.0.1:4243",
                    "Metadata": {"LastSuccess": "2014-08-01T14:09:40-03:00",
                                 "pool": "theonepool"},
                 "Status": "ready"},
                {"Address": "http://127.0.0.1:2375",
                 "Metadata": {"LastSuccess": "2014-08-01T14:09:40-03:00",
                              "pool": "theonepool"},
                 "Status": "ready"},
                {"Address": "http://myserver.com:2375",
                 "Metadata": {"LastSuccess": "2014-08-01T14:09:40-03:00",
                              "pool": "anotherpool"},
                 "Status": "ready"},
            ],
        }
        self.api_body = json.dumps(data)
        self.maxDiff = None
        self.request = RequestFactory().get("/")
        self.request.session = {"tsuru_token": "admin"}

    @httpretty.activate
    def test_get_pool_nodes_addrs(self):
        httpretty.register_uri(httpretty.GET, self.api_url, body=self.api_body)
        view = views.PoolMetric(request=self.request)
        addrs = view.get_pool_nodes("theonepool")
        self.assertEqual(addrs, ["128.0.0.1", "127.0.0.1"])

    @httpretty.activate
    @patch("requests.post")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_get_pool_metric(self, token_is_valid, post_mock):
        token_is_valid.return_value = True
        httpretty.register_uri(httpretty.GET, self.api_url, body=self.api_body)
        view = views.PoolMetric.as_view()
        request = RequestFactory().get("/ble/?metric=cpu_max&date_range=2h/h&interval=30m")
        request.session = {"tsuru_token": "token"}
        response = view(request, target="theonepool")

        self.assertEqual(response.status_code, 200)



from django.shortcuts import redirect
from django.core.urlresolvers import reverse
from django.contrib import messages
from django.views.generic import TemplateView
from django.template.response import TemplateResponse

from tsuru_dashboard import settings
from tsuru_dashboard.auth.views import LoginRequiredView

from .forms import TeamForm

import requests


class RemoveUser(LoginRequiredView):
    def get(self, request, *args, **kwargs):
        team_name = kwargs["team"]
        user = kwargs["user"]
        url = "{}/teams/{}/{}".format(settings.TSURU_HOST, team_name, user)
        response = requests.delete(url, headers=self.authorization)

        if response.status_code < 399:
            messages.success(self.request, u'User successfully removed!', fail_silently=True)
        else:
            messages.error(self.request, response.text, fail_silently=True)

        return redirect(reverse('team-info', args=[team_name]))


class AddUser(LoginRequiredView):
    def post(self, request, *args, **kwargs):
        team_name = kwargs["team"]
        user = request.POST["user"]
        url = "{0}/teams/{1}/{2}".format(settings.TSURU_HOST, team_name, user)
        response = requests.put(url, headers=self.authorization)
        if response.status_code != 200:
            messages.error(self.request, response.text, fail_silently=True)
        return redirect(reverse('team-info', args=[team_name]))


class Info(LoginRequiredView, TemplateView):
    template_name = "teams/info.html"

    def get_context_data(self, *args, **kwargs):
        context = super(Info, self).get_context_data(*args, **kwargs)
        url = "{0}/teams/{1}".format(settings.TSURU_HOST, kwargs["team"])
        response = requests.get(url, headers=self.authorization)
        context.update({"team": response.json()})
        return context


class List(LoginRequiredView, TemplateView):
    template_name = "teams/list.html"

    def get_context_data(self, *args, **kwargs):
        context = super(List, self).get_context_data(*args, **kwargs)

        url = "{}/teams".format(settings.TSURU_HOST)
        response = requests.get(url, headers=self.authorization)

        teams = []
        if response.status_code != 204:
            teams = response.json()

        context.update({"teams": teams})
        return context


class Remove(LoginRequiredView):
    def get(self, request, *args, **kwargs):
        team_name = kwargs["team"]
        auth = {'authorization': request.session.get('tsuru_token')}
        url = "{0}/teams/{1}".format(settings.TSURU_HOST, team_name)
        response = requests.delete(url, headers=auth)

        if response.status_code > 399:
            messages.error(self.request, u'Can not delete this team!', fail_silently=True)
        return redirect(reverse('team-list'))


class Add(LoginRequiredView):
    def get(self, request):
        return TemplateResponse(request, 'auth/team.html', {"form": TeamForm()})

    def post(self, request):
        form = TeamForm(request.POST)
        errors = ''
        if form.is_valid():
            url = '{}/teams'.format(settings.TSURU_HOST)
            response = requests.post(url, data={'name': form.data['name']}, headers=self.authorization)
            if response.status_code == 200:
                return redirect(reverse("team-list"))
            errors = response.content
        return TemplateResponse(request, 'auth/team.html', {'form': form, 'errors': errors})



from django.test import TestCase
from django.test.client import RequestFactory
from django.http import Http404

from tsuru_dashboard import settings
from tsuru_dashboard.apps.views import AppDetail

from mock import patch, Mock


class AppDetailTestCase(TestCase):
    @patch("requests.get")
    def setUp(self, requests_mock):
        request = RequestFactory().get("/")
        request.session = {"tsuru_token": "admin"}
        self.expected = {
            "name": "app1",
            "framework": "php",
            "repository": "git@git.com:php.git",
            "state": "dead",
            "units": [
                {"Ip": "10.10.10.10", "Status": "started", "ProcessName": "web"},
                {"Ip": "9.9.9.9", "Status": "stopped", "ProcessName": "worker"},
            ],
            "teams": ["tsuruteam", "crane"]
        }
        json_mock = Mock(status_code=200)
        json_mock.json.return_value = self.expected
        requests_mock.return_value = json_mock

        service_instances_mock = Mock()
        service_instances_mock.return_value = [{"service": "mongodb", "instances": ["mymongo"]}]

        self.old_service_instances = AppDetail.service_instances
        AppDetail.service_instances = service_instances_mock

        self.response = AppDetail.as_view()(request, app_name="app1")
        self.request = request

    def tearDown(self):
        AppDetail.service_instances = self.old_service_instances

    def test_should_use_detail_template(self):
        self.assertIn("apps/details.html", self.response.template_name)

    def test_should_get_the_app_info_from_tsuru(self):
        self.assertDictEqual(self.expected, self.response.context_data["app"])

    def test_service_instances_context(self):
        context = self.response.context_data
        service_instances = context["app"]["service_instances"]
        self.assertListEqual(service_instances, [{"name": "mymongo", "servicename": "mongodb"}])

    @patch('requests.get')
    def test_service_instances_empty(self, get):
        response = Mock(status_code=204)
        response.json.side_effect = ValueError
        get.return_value = response
        AppDetail.service_instances = self.old_service_instances
        app_detail = AppDetail()
        app_detail.request = self.request
        app_detail.service_instances("appname")
        get.assert_called_with(
            '{0}/services/instances?app=appname'.format(settings.TSURU_HOST),
            headers={'authorization': self.request.session.get('tsuru_token')})

    @patch("requests.get")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_not_found(self, token_is_valid, requests_mock):
        token_is_valid.return_value = True
        request = RequestFactory().get("/")
        request.session = {"tsuru_token": "admin"}
        requests_mock.return_value = Mock(status_code=404)

        service_instances_mock = Mock()
        service_instances_mock.return_value = [{"service": "mongodb", "instances": ["mymongo"]}]
        AppDetail.service_instances = service_instances_mock

        with self.assertRaises(Http404):
            AppDetail.as_view()(request, app_name="app1")



from django.conf.urls import url

import views


urlpatterns = [
    url(r'^app/(?P<target>[\w-]+)/$', views.AppMetric.as_view(), name='app-metric'),
    url(r'^component/(?P<target>[\w-]+)/$', views.ComponentMetric.as_view(), name='component-metric'),
    url(r'^node/(?P<target>[\w.-]+)/$', views.NodeMetric.as_view(), name='node-metric'),
    url(r'^pool/(?P<target>[\w-]+)/$', views.PoolMetric.as_view(), name='pool-metric')
]



import os

from django.core.wsgi import get_wsgi_application
from whitenoise.django import DjangoWhiteNoise


os.environ.setdefault("DJANGO_SETTINGS_MODULE", "abyss.settings")

application = get_wsgi_application()
application = DjangoWhiteNoise(application)
